from pydantic import BaseModel
from bifrostx.interface.base import BaseInterface
from bifrostx.utils.logger import logger
from .profile import AdapterProfile
from typing import Dict, List, Type
from bifrostx.config import Config


class AdapterInfo(BaseModel):
    module_name: str
    adapter: Type[BaseInterface]
    profile: AdapterProfile

    @property
    def instance_configs(self):
        instances_configs = Config.get_extension_config(
            f"Adapters.{self.module_name}", instances=True
        )
        return instances_configs


class AdapterRegister:
    adapters: Dict[str, BaseInterface] = {}

    @classmethod
    def register(cls, module_name):
        try:
            from bifrostx.interface.register import InterfaceRegister

            profile = AdapterProfile.load_by_module_name(module_name)
            adapter = profile.load_enter_class()
            adapter_info = AdapterInfo(
                module_name=module_name, adapter=adapter, profile=profile
            )
            cls.adapters[module_name] = adapter_info
            for interface_module_name in profile.implements:
                if interface_module_name.interface not in InterfaceRegister.interfaces:
                    raise ValueError(
                        f"没有找到依赖的Interface({interface_module_name.interface})"
                    )
                InterfaceRegister.interfaces[interface_module_name.interface].adapters[
                    module_name
                ] = adapter_info
            logger.info(f"Load Adapter [{module_name}] Success")
        except Exception as ex:
            logger.warning(f"Load Adapter [{module_name}] Error: {ex}")

    @classmethod
    def get_adapters(cls) -> List[AdapterInfo]:
        return list(cls.adapters.values())



from bifrostx.core.profile import BaseProfile
from pydantic import model_validator, BaseModel
from bifrostx.interface.profile import InterfaceProfile
from typing import List


class ImplementInterface(BaseModel):
    interface: str
    interface_version: str


class AdapterProfile(BaseProfile):
    implements: List[ImplementInterface]
    enter_class: str = "Adapter"

    @classmethod
    def load_by_module_name(cls, model_name):
        return super().load_by_module_name(f"Adapters.{model_name}")

    @model_validator(mode="after")
    def validate_interface(self):
        for implement in self.implements:
            try:
                interface_info = InterfaceProfile.load_by_module_name(
                    implement.interface
                )
            except ImportError:
                raise ValueError(
                    f"Adapter[{self.display_name}]所依赖的Interface[{implement.interface}]不存在"
                )
            if interface_info.version == implement.interface_version:
                return self
            raise ValueError(
                f"Adapter[{self.display_name}]所依赖的Interface[{implement.interface}]版本不一致"
            )



from pydantic import BaseModel


class BaseComponent:
    instance_config_schema: BaseModel = None

    def __init__(self, instance_config):
        self.instance_config = (
            self.instance_config_schema(**instance_config)
            if self.instance_config_schema
            else None
        )



from pydantic import BaseModel
from .base import BaseInterface
from .profile import InterfaceProfile
from typing import Dict, List, Type, Union
from bifrostx.utils.logger import logger
from bifrostx.adapter.register import AdapterInfo


class InterfaceInfo(BaseModel):
    module_name: str
    interface: Type[BaseInterface]
    profile: InterfaceProfile
    adapters: Dict[str, AdapterInfo] = {}

    def get_adapter_instance(
        self, adapter_name=None, instance_id=None
    ) -> BaseInterface:
        if not adapter_name:
            for adapter in self.adapters.values():
                if adapter.instance_configs:
                    if not instance_id:
                        return adapter.adapter(
                            list(adapter.instance_configs.values())[0]
                        )
                    elif instance_id in adapter.instance_configs:
                        return adapter.adapter(adapter.instance_configs[instance_id])
            raise ValueError(f"未找到 Interface[{self.module_name}] 的实例")
        adapter_info = self.adapters.get(adapter_name)
        if not adapter_info:
            raise ValueError(
                f"未找到 Interface[{self.module_name}] 的 Adapter[{adapter_name}]"
            )
        if instance_id:
            instance_config = adapter_info.instance_configs.get(instance_id)
            if not instance_config:
                raise ValueError(
                    f"未找到 Interface[{self.module_name}] 的 Adapter[{adapter_name}] 的实例[{instance_id}]"
                )
            return adapter_info.adapter(instance_config)
        else:
            instance_configs = list(adapter_info.instance_configs.values())
            if not instance_configs:
                raise ValueError(
                    f"未找到 Interface[{self.module_name}] 的 Adapter[{adapter_name}] 的实例"
                )
            return adapter_info.adapter(instance_configs[0])


class InterfaceRegister:
    interfaces: Dict[str, InterfaceInfo] = {}

    @classmethod
    def register(cls, module_name: str):
        try:
            profile = InterfaceProfile.load_by_module_name(module_name)
            interface = profile.load_enter_class()
            cls.interfaces[module_name] = InterfaceInfo(
                module_name=module_name, interface=interface, profile=profile
            )
            logger.info(f"Load Interface [{module_name}] Success")
        except Exception as ex:
            logger.warning(f"Load Interface [{module_name}] Error: {ex}")

    @classmethod
    def get_interfaces(cls) -> List[InterfaceInfo]:
        return list(cls.interfaces.values())

    @classmethod
    def get_interface(cls, module: Union[str, Type]) -> InterfaceInfo:
        module_name = module
        if not isinstance(module, str):
            module_name = module.__name__.split(".")[1]
        module_name = (
            module_name
            if len(module_name.split(".")) == 1
            else module_name.split(".")[1]
        )
        return cls.interfaces.get(module_name)



from pydantic import BaseModel, field_validator, model_validator
from bifrostx import __version__ as bifrostx_version
from distutils.version import LooseVersion
import re
from pathlib import Path
import tomli
from importlib import import_module
from typing import List
import pkg_resources
from bifrostx.utils.logger import logger
import sys
import subprocess
from bifrostx.config import Config


class BaseProfile(BaseModel):
    module_name: str = ""
    version: str
    display_name: str
    bifrostx_version: str = ""
    dependencies: List[str] = []
    enter_class: str

    @field_validator("version")
    @classmethod
    def validate_version(cls, value: str):
        if re.match(r"^[0-9]+\.[0-9]+\.[0-9]+$", value):
            return value
        raise ValueError(f"版本号格式错误")

    @field_validator("bifrostx_version")
    @classmethod
    def validate_bifrostx_version(cls, value: str):
        if value:
            current_version = LooseVersion(bifrostx_version)
            if current_version >= LooseVersion(value):
                return value
            raise ValueError(f"拓展需求版本与当前不匹配")
        return value

    @classmethod
    def load_by_module(cls, module):
        file = Path(module.__file__).parent.joinpath("bifrostx.toml")
        if not file.exists():
            raise ValueError("未找到bifrostx.toml")
        info = tomli.loads(file.read_text(encoding="utf-8"))
        rel = cls(**info)
        rel.module_name = module.__name__
        return rel

    @classmethod
    def load_by_module_name(cls, module_name):
        file = (
            Path(Config.EXTENSION_DIR)
            .joinpath(*module_name.split("."))
            .joinpath("bifrostx.toml")
        )
        if not file.exists():
            raise ValueError("未找到bifrostx.toml")
        info = tomli.loads(file.read_text(encoding="utf-8"))
        rel = cls(**info)
        rel.module_name = module_name
        return rel

    def load_enter_class(self):
        enter_class_path = self.enter_class.strip().strip(".").rsplit(":", 1)
        if len(enter_class_path) == 1:
            module = import_module(self.module_name)
        else:
            module = import_module(f"{self.module_name}.{enter_class_path[0]}")
        if hasattr(module, enter_class_path[-1]):
            return getattr(module, enter_class_path[-1])
        else:
            raise ValueError(f"Enter Class: {self.module_name}.{self.enter_class}不存在")

    @model_validator(mode="after")
    def validate_dependencies(self):
        if not self.dependencies:
            return self
        for dependency in self.dependencies:
            rel = re.search(
                r"^([^>=<]*)([>=<]*)([^>=<]*)$", dependency.replace(" ", "")
            )
            if rel:
                key, _, ver = rel.groups()
                # TODO: 检查依赖版本冲突
                if key not in [pkg.key for pkg in pkg_resources.working_set]:
                    subprocess.check_call(
                        [sys.executable, "-m", "pip", "install", dependency]
                    )
            else:
                logger.warning(f"{self.display_name} 依赖包 {dependency} 格式不正确")
        return self



from pydantic import BaseModel
from .base import BaseComponent
from bifrostx.interface.register import InterfaceRegister, InterfaceInfo
from .profile import ComponentProfile
from typing import Dict, List, Type
from bifrostx.utils.logger import logger


class ComponentInfo(BaseModel):
    module_name: str
    component: Type[BaseComponent]
    profile: ComponentProfile
    interfaces: Dict[str, InterfaceInfo]


class ComponentRegister:
    components: Dict[str, ComponentInfo] = {}

    @classmethod
    def register(cls, module_name):
        try:
            profile = ComponentProfile.load_by_module_name(module_name)
            component = profile.load_enter_class()
            interfaces = {
                i.interface: InterfaceRegister.interfaces[i.interface]
                for i in profile.references
            }
            component_info = ComponentInfo(
                module_name=module_name,
                component=component,
                profile=profile,
                interfaces=interfaces,
            )
            cls.components[module_name] = component_info
            logger.info(f"Load Component [{module_name}] Success")

        except Exception as ex:
            logger.warning(f"Load Component [{module_name}] Error: {ex}")

    @classmethod
    def get_components(cls) -> List[ComponentInfo]:
        return list(cls.components.values())
